/*
 * Copyright (c) 2014, 2015, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.jmh.it.profilers.order;

import org.openjdk.jmh.results.AggregationPolicy;
import org.openjdk.jmh.results.Aggregator;
import org.openjdk.jmh.results.Result;
import org.openjdk.jmh.results.ResultRole;

import java.util.Collection;

public class ProfilerTimestamp extends Result<ProfilerTimestamp> {

    public ProfilerTimestamp(String label, double ts) {
        super(ResultRole.SECONDARY, label, of(ts), "N/A", AggregationPolicy.MAX);
    }

    @Override
    protected Aggregator<ProfilerTimestamp> getThreadAggregator() {
        return new LogAggregator();
    }

    @Override
    protected Aggregator<ProfilerTimestamp> getIterationAggregator() {
        return new LogAggregator();
    }

    public static class LogAggregator implements Aggregator<ProfilerTimestamp> {
        @Override
        public ProfilerTimestamp aggregate(Collection<ProfilerTimestamp> results) {
            String label = null;
            double d = Double.MIN_VALUE;
            for (ProfilerTimestamp r : results) {
                label = r.label;
                d = Math.max(d, r.getScore());
            }
            return new ProfilerTimestamp(label, d);
        }
    }

}
