/*
 * Copyright (c) 2005, 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.jmh.generators.core;

class BenchmarkInfo {
    public final String userClassQName;
    public final String generatedClassQName;
    public final String generatedPackageName;
    public final String generatedClassName;
    public final MethodGroup methodGroup;

    public BenchmarkInfo(String userClassQName, String generatedPackageName, String generatedClassName, MethodGroup methodGroup) {
        this.userClassQName = userClassQName;
        this.generatedPackageName = generatedPackageName;
        this.generatedClassName = generatedClassName;
        this.generatedClassQName = generatedPackageName + "." + generatedClassName;
        this.methodGroup = methodGroup;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        BenchmarkInfo that = (BenchmarkInfo) o;

        if (!methodGroup.equals(that.methodGroup)) return false;
        if (!userClassQName.equals(that.userClassQName)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = userClassQName.hashCode();
        result = 31 * result + methodGroup.hashCode();
        return result;
    }
}
