/*
 * Copyright (c) 2005, 2013, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.jmh.it.ccontrol;

import org.junit.Assert;
import org.junit.Test;
import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.CompilerControl;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.it.Fixtures;
import org.openjdk.jmh.results.RunResult;
import org.openjdk.jmh.runner.Runner;
import org.openjdk.jmh.runner.RunnerException;
import org.openjdk.jmh.runner.options.Options;
import org.openjdk.jmh.runner.options.OptionsBuilder;
import org.openjdk.jmh.runner.options.TimeValue;

import java.util.Collection;

public class CompilerControlExcludeActualTest {

    @CompilerControl(CompilerControl.Mode.EXCLUDE)
    public void strawMethod() {

    }

    public void trampoline() {
        strawMethod();
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.EXCLUDE)
    public void compilerControlSpecimen() {
        trampoline();
    }

    @Test
    public void testBenchmark() throws RunnerException {
        for (Mode mode : Mode.values()) {
            if (mode == Mode.All) continue;

            Options opts = new OptionsBuilder()
                    .include(Fixtures.getTestMask(this.getClass()))
                    .mode(mode)
                    .shouldFailOnError(true)
                    .addProfiler(LogConsumeProfiler.class)
                    .measurementIterations(mode == Mode.SingleShotTime ? 200_000 : 1)
                    .measurementTime(TimeValue.seconds(5))
                    .warmupIterations(0)
                    .forks(1)
                    .jvmArgsPrepend("-XX:CICompilerCount=2") // need to serialize the output properly
                    .build();
            RunResult runResult = new Runner(opts).runSingle();

            Collection<String> log = CompilerControlUtils.getLog(runResult);

            if (CompilerControlUtils.check(log, "@", "callee")) { // Poor man's check -XX:+PrintInlining works
                Assert.assertTrue("Failed with " + mode,
                         CompilerControlUtils.check(log, this.getClass().getName() + "::compilerControlSpecimen", "excluded by") ||
                         CompilerControlUtils.check(log, this.getClass().getName() + "::compilerControlSpecimen", "not compilable") ||
                         CompilerControlUtils.check(log, this.getClass().getName() + "::compilerControlSpecimen", "disallowed by CompileCommand")
                );
                Assert.assertTrue("Failed with " + mode,
                        CompilerControlUtils.check(log, this.getClass().getName() + "::strawMethod", "excluded by") ||
                        CompilerControlUtils.check(log, this.getClass().getName() + "::strawMethod", "not compilable") ||
                        CompilerControlUtils.check(log, this.getClass().getName() + "::strawMethod", "disallowed by CompileCommand")
                );
            }
        }
    }



}
