/*
 * Copyright (c) 2005, 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.jmh.it.ccontrol;

import org.junit.Assert;
import org.junit.Test;
import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.CompilerControl;
import org.openjdk.jmh.annotations.CompilerControl.Mode;
import org.openjdk.jmh.it.Fixtures;
import org.openjdk.jmh.runner.RunnerException;

public class CompilerControlNestedTest {

    @CompilerControl(Mode.INLINE)
    public static class SpecimenClass {
        void sampleMethod() {}
    }


    public static class SpecimenMethod {
        @CompilerControl(Mode.INLINE)
        void sampleMethod() {}
    }

    @Benchmark
    public void dummyTest() {
        Fixtures.work();
    }

    @Test
    public void testClass() throws RunnerException {
        String className = this.getClass().getName().replace(".", "/");
        Assert.assertTrue("Has hint on nested class",
                CompilerControlUtils.hasHint(Mode.INLINE.command(), className, "$SpecimenClass.*"));
    }

    @Test
    public void testMethod() throws RunnerException {
        String className = this.getClass().getName().replace(".", "/");
        Assert.assertTrue("Has hint on nested class",
                CompilerControlUtils.hasHint(Mode.INLINE.command(), className, "$SpecimenMethod", "sampleMethod"));
    }

}
