/*
 * Copyright (c) 2014, 2015, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.jmh.results;

import java.util.ArrayList;
import java.util.Collection;

public class BenchmarkTaskResult {

    /**
     * Total benchmark ops done.
     */
    private final long allOperations;

    /**
     * Measured benchmark ops done: total without sync iterations.
     */
    private final long measuredOperations;

    private final Collection<Result> results;

    public BenchmarkTaskResult(long allOperations, long measuredOperations) {
        this.allOperations = allOperations;
        this.measuredOperations = measuredOperations;
        this.results = new ArrayList<>();
    }

    public void add(Result result) {
        results.add(result);
    }

    public Collection<Result> getResults() {
        return results;
    }

    public long getAllOps() {
        return allOperations;
    }

    public long getMeasuredOps() {
        return measuredOperations;
    }
}
